package gregtech.api.items;

import codechicken.lib.render.DigIconParticle;
import gregtech.api.util.GT_LanguageManager;
import gregtech.common.render.RenderBlocks;
import net.minecraft.block.Block;
import net.minecraft.block.material.Material;
import net.minecraft.block.properties.IProperty;
import net.minecraft.block.properties.PropertyInteger;
import net.minecraft.block.state.BlockStateContainer;
import net.minecraft.block.state.IBlockState;
import net.minecraft.client.particle.ParticleDigging;
import net.minecraft.client.particle.ParticleManager;
import net.minecraft.client.renderer.texture.TextureAtlasSprite;
import net.minecraft.client.renderer.texture.TextureMap;
import net.minecraft.entity.EntityLivingBase;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.item.ItemBlock;
import net.minecraft.item.ItemStack;
import net.minecraft.util.EnumBlockRenderType;
import net.minecraft.util.EnumFacing;
import net.minecraft.util.math.AxisAlignedBB;
import net.minecraft.util.math.BlockPos;
import net.minecraft.util.math.RayTraceResult;
import net.minecraft.world.IBlockAccess;
import net.minecraft.world.World;
import net.minecraft.world.WorldServer;
import net.minecraftforge.fml.common.registry.GameRegistry;
import net.minecraftforge.fml.relauncher.Side;
import net.minecraftforge.fml.relauncher.SideOnly;

import java.util.Random;

import static gregtech.api.enums.GT_Values.W;

public abstract class GT_Generic_Block extends Block {

    public static IProperty<Integer> METADATA = PropertyInteger.func_177719_a("metadata", 0, 15);

    public float minX, minY, minZ, maxX, maxY, maxZ;
    public final String mUnlocalizedName;

    protected GT_Generic_Block(Class<? extends ItemBlock> aItemClass, String aName, Material aMaterial) {
        super(aMaterial);
        func_149663_c(mUnlocalizedName = aName);
        GameRegistry.registerBlock(this, aItemClass, func_149739_a());
        GT_LanguageManager.addStringLocalization(func_149739_a() + "." + W + ".name", "Unnamed");
        setBlockBounds(0.0f, 0.0f, 0.0f, 1.0f, 1.0f, 1.0f);
    }

    @Override
    public boolean addLandingEffects(IBlockState state, WorldServer worldObj, BlockPos blockPosition, IBlockState iblockstate, EntityLivingBase entity, int numberOfParticles) {
        return true;
    }

    @Override
    @SideOnly(Side.CLIENT)
    public boolean addHitEffects(IBlockState state, World worldObj, RayTraceResult target, ParticleManager manager) {
        addBlockHitEffects(worldObj, target.func_178782_a() != null ? target.func_178782_a() : BlockPos.field_177992_a,
                target.field_178784_b != null ? target.field_178784_b : EnumFacing.UP, worldObj.field_73012_v, manager);
        return true;
    }

    @Override
    @SideOnly(Side.CLIENT)
    public boolean addDestroyEffects(World world, BlockPos pos, ParticleManager manager) {
        int count = 14 + world.field_73012_v.nextInt(10);
        for(int i = 0; i < count; i++) {
            addBlockHitEffects(world, pos, EnumFacing.UP, world.field_73012_v, manager);
        }
        return true;
    }

    @SideOnly(Side.CLIENT)
    public void addBlockHitEffects(World worldObj, BlockPos pos, EnumFacing side, Random rand, ParticleManager manager)
    {
        IBlockState iblockstate = worldObj.func_180495_p(pos);

        if (iblockstate.func_185901_i() != EnumBlockRenderType.INVISIBLE) {
            int i = pos.func_177958_n();
            int j = pos.func_177956_o();
            int k = pos.func_177952_p();
            float f = 0.1F;
            AxisAlignedBB axisalignedbb = iblockstate.func_185900_c(worldObj, pos);
            double d0 = (double) i + rand.nextDouble() * (axisalignedbb.field_72336_d - axisalignedbb.field_72340_a - 0.20000000298023224D) + 0.10000000149011612D + axisalignedbb.field_72340_a;
            double d1 = (double) j + rand.nextDouble() * (axisalignedbb.field_72337_e - axisalignedbb.field_72338_b - 0.20000000298023224D) + 0.10000000149011612D + axisalignedbb.field_72338_b;
            double d2 = (double) k + rand.nextDouble() * (axisalignedbb.field_72334_f - axisalignedbb.field_72339_c - 0.20000000298023224D) + 0.10000000149011612D + axisalignedbb.field_72339_c;

            if (side == EnumFacing.DOWN) {
                d1 = (double) j + axisalignedbb.field_72338_b - 0.10000000149011612D;
            }

            if (side == EnumFacing.UP) {
                d1 = (double) j + axisalignedbb.field_72337_e + 0.10000000149011612D;
            }

            if (side == EnumFacing.NORTH) {
                d2 = (double) k + axisalignedbb.field_72339_c - 0.10000000149011612D;
            }

            if (side == EnumFacing.SOUTH) {
                d2 = (double) k + axisalignedbb.field_72334_f + 0.10000000149011612D;
            }

            if (side == EnumFacing.WEST) {
                d0 = (double) i + axisalignedbb.field_72340_a - 0.10000000149011612D;
            }

            if (side == EnumFacing.EAST) {
                d0 = (double) i + axisalignedbb.field_72336_d + 0.10000000149011612D;
            }


            TextureAtlasSprite sprite = getParticleSprite(worldObj, pos, side);
            if (sprite != null) {
                manager.func_78873_a((new DigIconParticle(worldObj, d0, d1, d2, 0.0D, 0.0D, 0.0D, sprite)).func_70543_e(0.2F).func_70541_f(0.6F));
            }
        }
    }

    @SideOnly(Side.CLIENT)
    public TextureAtlasSprite getParticleSprite(World worldObj, BlockPos aPos, EnumFacing side) {
        return getWorldIcon(worldObj, aPos, worldObj.func_180495_p(aPos), side);
    }
    
    @SideOnly(Side.CLIENT)
    public void registerIcons(TextureMap textureMap) {}

    @SideOnly(Side.CLIENT)
    public int getColorMultiplier(IBlockAccess aWorld, BlockPos aPos, IBlockState aState) {
        return 0xFFFFFFFF;
    }

    @SideOnly(Side.CLIENT)
    public TextureAtlasSprite getWorldIcon(IBlockAccess aWorld, BlockPos aPos, IBlockState aState, EnumFacing aSide) {
        return getIcon(aSide, aState.func_177229_b(METADATA));
    }

    @SideOnly(Side.CLIENT)
    public TextureAtlasSprite getItemIcon(ItemStack itemStack, EnumFacing aSide) {
        return getIcon(aSide, itemStack.func_77952_i());
    }

    public TextureAtlasSprite getIcon(EnumFacing aSide, int metadata) {
        return null;
    }

    public void setBlockBounds(float minX, float minY, float minZ, float maxX, float maxY, float maxZ) {
        this.minX = minX;
        this.minY = minY;
        this.minZ = minZ;
        this.maxX = maxX;
        this.maxY = maxY;
        this.maxZ = maxZ;
    }

    public void setBlockBoundsForItemRender() {}
    public void setBlockBoundsBasedOnState(IBlockAccess world, BlockPos pos, IBlockState state) {}

    @Override
    protected BlockStateContainer func_180661_e() {
        return new BlockStateContainer(this, METADATA);
    }

    @Override
    public IBlockState func_176203_a(int meta) {
        return func_176223_P().func_177226_a(METADATA, meta);
    }

    @Override
    public int func_176201_c(IBlockState state) {
        return state.func_177229_b(METADATA);
    }

    @Override
    public ItemStack getPickBlock(IBlockState state, RayTraceResult target, World world, BlockPos pos, EntityPlayer player) {
        return new ItemStack(this, 1, state.func_177229_b(METADATA));
    }

    @Override
    public String func_149739_a() {
        return mUnlocalizedName;
    }

    @Override
    public EnumBlockRenderType func_149645_b(IBlockState state) {
        return RenderBlocks.INSTANCE.renderType;
    }
}
