package gregtech.common.render;

import codechicken.lib.render.block.BlockRenderingRegistry;
import codechicken.lib.render.block.ICCBlockRenderer;
import gregtech.api.enums.Materials;
import gregtech.api.enums.StoneTypes;
import gregtech.api.enums.TextureSet;
import gregtech.api.enums.Textures;
import gregtech.api.interfaces.ITexture;
import gregtech.api.items.GT_Generic_Block;
import gregtech.common.blocks.GT_Block_GeneratedOres;
import net.minecraft.block.state.IBlockState;
import net.minecraft.client.Minecraft;
import net.minecraft.client.renderer.Tessellator;
import net.minecraft.client.renderer.VertexBuffer;
import net.minecraft.client.renderer.texture.TextureAtlasSprite;
import net.minecraft.client.renderer.texture.TextureMap;
import net.minecraft.client.renderer.vertex.DefaultVertexFormats;
import net.minecraft.init.Blocks;
import net.minecraft.item.ItemBlock;
import net.minecraft.item.ItemStack;
import net.minecraft.util.EnumBlockRenderType;
import net.minecraft.util.EnumFacing;
import net.minecraft.util.math.BlockPos;
import net.minecraft.world.IBlockAccess;
import org.lwjgl.opengl.GL11;

public class RenderBlocks implements ICCBlockRenderer {

    public static RenderBlocks INSTANCE = new RenderBlocks();

    public float minX, minY, minZ, maxX, maxY, maxZ;
    public boolean flip = true;

    public EnumBlockRenderType renderType;
    private BlockPos.MutableBlockPos t = new BlockPos.MutableBlockPos();

    public void init() {
        renderType = BlockRenderingRegistry.createRenderType("GT_SIMPLE_BLOCK");
        BlockRenderingRegistry.registerRenderer(renderType, INSTANCE);
        setBlockBounds(0.0f, 0.0f, 0.0f, 1.0f, 1.0f, 1.0f);
    }

    @Override
    public void handleRenderBlockDamage(IBlockAccess world, BlockPos pos, IBlockState state, TextureAtlasSprite sprite, VertexBuffer buffer) {
        IBlockState stone = Blocks.field_150348_b.func_176223_P();
        Minecraft.func_71410_x().func_175602_ab().func_175020_a(stone, pos, sprite, world);
    }

    @Override
    public boolean renderBlock(IBlockAccess world, BlockPos pos, IBlockState state, VertexBuffer buffer) {
        double x = pos.func_177958_n();
        double y = pos.func_177956_o();
        double z = pos.func_177952_p();


        GT_Generic_Block aOres = (GT_Generic_Block) state.func_177230_c();
        aOres.setBlockBoundsBasedOnState(world, pos, state);
        setRenderBoundsFromBlock1(aOres);

        int lightmap;
        int color;
        TextureAtlasSprite sprite;

        if(state.func_185894_c(world, pos, EnumFacing.UP)) {
            if((sprite = aOres.getWorldIcon(world, pos, state, EnumFacing.UP)) != null) {
                color = aOres.getColorMultiplier(world, pos, state);
                t.func_181079_c(pos.func_177958_n(), pos.func_177956_o() + 1, pos.func_177952_p());
                lightmap = world.func_180495_p(t).func_185889_a(world, t);
                renderYPos(sprite, x, y, z, color, lightmap, buffer);
            }
        }

        if(state.func_185894_c(world, pos, EnumFacing.DOWN)) {
            if((sprite = aOres.getWorldIcon(world, pos, state, EnumFacing.DOWN)) != null) {
                color = aOres.getColorMultiplier(world, pos, state);
                t.func_181079_c(pos.func_177958_n(), pos.func_177956_o() - 1, pos.func_177952_p());
                lightmap = world.func_180495_p(t).func_185889_a(world, t);
                renderYNeg(sprite, x, y, z, color, lightmap, buffer);
            }
        }

        if(state.func_185894_c(world, pos, EnumFacing.NORTH)) {
            if((sprite = aOres.getWorldIcon(world, pos, state, EnumFacing.NORTH)) != null) {
                color = aOres.getColorMultiplier(world, pos, state);
                t.func_181079_c(pos.func_177958_n(), pos.func_177956_o(), pos.func_177952_p() - 1);
                lightmap = world.func_180495_p(t).func_185889_a(world, t);
                renderZNeg(sprite, x, y, z, color, lightmap, buffer);
            }
        }

        if(state.func_185894_c(world, pos, EnumFacing.SOUTH)) {
            if((sprite = aOres.getWorldIcon(world, pos, state, EnumFacing.SOUTH)) != null) {
                color = aOres.getColorMultiplier(world, pos, state);
                t.func_181079_c(pos.func_177958_n(), pos.func_177956_o(), pos.func_177952_p() + 1);
                lightmap = world.func_180495_p(t).func_185889_a(world, t);
                renderZPos(sprite, x, y, z, color, lightmap, buffer);
            }
        }

        if(state.func_185894_c(world, pos, EnumFacing.WEST)) {
            if((sprite = aOres.getWorldIcon(world, pos, state, EnumFacing.WEST)) != null) {
                color = aOres.getColorMultiplier(world, pos, state);
                t.func_181079_c(pos.func_177958_n() - 1, pos.func_177956_o(), pos.func_177952_p());
                lightmap = world.func_180495_p(t).func_185889_a(world, t);
                renderXNeg(sprite, x, y, z, color, lightmap, buffer);
            }
        }

        if(state.func_185894_c(world, pos, EnumFacing.EAST)) {
            if((sprite = aOres.getWorldIcon(world, pos, state, EnumFacing.EAST)) != null) {
                color = aOres.getColorMultiplier(world, pos, state);
                t.func_181079_c(pos.func_177958_n() + 1, pos.func_177956_o(), pos.func_177952_p());
                lightmap = world.func_180495_p(t).func_185889_a(world, t);
                renderXPos(sprite, x, y, z, color, lightmap, buffer);
            }
        }

        return true;
    }
    
    public void renderOresBlock(IBlockAccess world, BlockPos pos, IBlockState state, VertexBuffer buffer) {
        double x = pos.func_177958_n();
        double y = pos.func_177956_o();
        double z = pos.func_177952_p();

        GT_Block_GeneratedOres aOres = (GT_Block_GeneratedOres) state.func_177230_c();
        Materials mats = aOres.getMaterialSafe(state);
        boolean small = aOres.mSmall;
        int lightmap;

        int color = ITexture.color(mats.mRGBa, false);
        TextureAtlasSprite sprite1 = aOres.getStoneTypeSafe(state).mIconContainer.getIcon();
        TextureAtlasSprite sprite2 = mats.mIconSet.mTextures[small ? TextureSet.INDEX_oreSmall : TextureSet.INDEX_ore].getIcon();

        if(state.func_185894_c(world, pos, EnumFacing.UP)) {
            t.func_181079_c(pos.func_177958_n(), pos.func_177956_o() + 1, pos.func_177952_p());
            lightmap = world.func_180495_p(t).func_185889_a(world, t);
            renderYPos(sprite1, x, y, z, 0xFFFFFFFF, lightmap, buffer);
            renderYPos(sprite2, x, y, z, color, lightmap, buffer);
        }

        if(state.func_185894_c(world, pos, EnumFacing.DOWN)) {
            t.func_181079_c(pos.func_177958_n(), pos.func_177956_o() - 1, pos.func_177952_p());
            lightmap = world.func_180495_p(t).func_185889_a(world, t);
            renderYNeg(sprite1, x, y, z, 0xFFFFFFFF, lightmap, buffer);
            renderYNeg(sprite2, x, y, z, color, lightmap, buffer);
        }

        if(state.func_185894_c(world, pos, EnumFacing.NORTH)) {
            t.func_181079_c(pos.func_177958_n(), pos.func_177956_o(), pos.func_177952_p() - 1);
            lightmap = world.func_180495_p(t).func_185889_a(world, t);
            renderZNeg(sprite1, x, y, z, 0xFFFFFFFF, lightmap, buffer);
            renderZNeg(sprite2, x, y, z, color, lightmap, buffer);
        }

        if(state.func_185894_c(world, pos, EnumFacing.SOUTH)) {
            t.func_181079_c(pos.func_177958_n(), pos.func_177956_o(), pos.func_177952_p() + 1);
            lightmap = world.func_180495_p(t).func_185889_a(world, t);
            renderZPos(sprite1, x, y, z, 0xFFFFFFFF, lightmap, buffer);
            renderZPos(sprite2, x, y, z, color, lightmap, buffer);
        }

        if(state.func_185894_c(world, pos, EnumFacing.WEST)) {
            t.func_181079_c(pos.func_177958_n() - 1, pos.func_177956_o(), pos.func_177952_p());
            lightmap = world.func_180495_p(t).func_185889_a(world, t);
            renderXNeg(sprite1, x, y, z, 0xFFFFFFFF, lightmap, buffer);
            renderXNeg(sprite2, x, y, z, color, lightmap, buffer);
        }

        if(state.func_185894_c(world, pos, EnumFacing.EAST)) {
            t.func_181079_c(pos.func_177958_n() + 1, pos.func_177956_o(), pos.func_177952_p());
            lightmap = world.func_180495_p(t).func_185889_a(world, t);
            renderXPos(sprite1, x, y, z, 0xFFFFFFFF, lightmap, buffer);
            renderXPos(sprite2, x, y, z, color, lightmap, buffer);
        }

    }

    public void renderBlockAsItem(ItemStack itemStack) {

        Tessellator tes = Tessellator.func_178181_a();
        VertexBuffer buf = tes.func_178180_c();

        GT_Generic_Block aBlock = (GT_Generic_Block) ((ItemBlock) itemStack.func_77973_b()).field_150939_a;
        aBlock.setBlockBoundsForItemRender();
        setRenderBoundsFromBlock1(aBlock);

        TextureAtlasSprite sprite;

        buf.func_181668_a(GL11.GL_QUADS, DefaultVertexFormats.field_176599_b);
        if((sprite = aBlock.getItemIcon(itemStack, EnumFacing.DOWN)) != null) {
            renderYNegItem(sprite, 0, 0, 0, 0xFFFFFFFF, buf);
        }
        tes.func_78381_a();

        buf.func_181668_a(GL11.GL_QUADS, DefaultVertexFormats.field_176599_b);
        if((sprite = aBlock.getItemIcon(itemStack, EnumFacing.UP)) != null) {
            renderYPosItem(sprite, 0, 0, 0, 0xFFFFFFFF, buf);
        }
        tes.func_78381_a();

        buf.func_181668_a(GL11.GL_QUADS, DefaultVertexFormats.field_176599_b);
        if((sprite = aBlock.getItemIcon(itemStack, EnumFacing.NORTH)) != null) {
            renderZNegItem(sprite, 0, 0, 0, 0xFFFFFFFF, buf);
        }
        tes.func_78381_a();

        buf.func_181668_a(GL11.GL_QUADS, DefaultVertexFormats.field_176599_b);
        if((sprite = aBlock.getItemIcon(itemStack, EnumFacing.SOUTH)) != null) {
            renderZPosItem(sprite, 0, 0, 0, 0xFFFFFFFF, buf);
        }
        tes.func_78381_a();

        buf.func_181668_a(GL11.GL_QUADS, DefaultVertexFormats.field_176599_b);
        if((sprite = aBlock.getItemIcon(itemStack, EnumFacing.EAST)) != null) {
            renderXPosItem(sprite, 0, 0, 0, 0xFFFFFFFF, buf);
        }
        tes.func_78381_a();

        buf.func_181668_a(GL11.GL_QUADS, DefaultVertexFormats.field_176599_b);
        if((sprite = aBlock.getItemIcon(itemStack, EnumFacing.WEST)) != null) {
            renderXNegItem(sprite, 0, 0, 0, 0xFFFFFFFF, buf);
        }
        tes.func_78381_a();
    }

    public void renderOresAsItem(ItemStack stack) {
        Tessellator tes = Tessellator.func_178181_a();
        VertexBuffer buf = tes.func_178180_c();

        GT_Block_GeneratedOres aOres = (GT_Block_GeneratedOres) ((ItemBlock) stack.func_77973_b()).field_150939_a;
        Materials mats = aOres.getMaterialSafe(stack);
        boolean small = aOres.mSmall;

        int color = ITexture.color(mats.mRGBa, false);
        TextureAtlasSprite sprite1 = aOres.getStoneTypeSafe(stack).mIconContainer.getIcon();
        TextureAtlasSprite sprite2 = mats.mIconSet.mTextures[small ? TextureSet.INDEX_oreSmall : TextureSet.INDEX_ore].getIcon();

        buf.func_181668_a(GL11.GL_QUADS, DefaultVertexFormats.field_176599_b);
        renderYNegItem(sprite1, 0, 0, 0, 0xFFFFFFFF, buf);
        renderYNegItem(sprite2, 0, 0, 0, color, buf);
        tes.func_78381_a();

        buf.func_181668_a(GL11.GL_QUADS, DefaultVertexFormats.field_176599_b);
        renderYPosItem(sprite1, 0, 0, 0, 0xFFFFFFFF, buf);
        renderYPosItem(sprite2, 0, 0, 0, color, buf);
        tes.func_78381_a();

        buf.func_181668_a(GL11.GL_QUADS, DefaultVertexFormats.field_176599_b);
        renderZNegItem(sprite1, 0, 0, 0, 0xFFFFFFFF, buf);
        renderZNegItem(sprite2, 0, 0, 0, color, buf);
        tes.func_78381_a();

        buf.func_181668_a(GL11.GL_QUADS, DefaultVertexFormats.field_176599_b);
        renderZPosItem(sprite1, 0, 0, 0, 0xFFFFFFFF, buf);
        renderZPosItem(sprite2, 0, 0, 0, color, buf);
        tes.func_78381_a();

        buf.func_181668_a(GL11.GL_QUADS, DefaultVertexFormats.field_176599_b);
        renderXPosItem(sprite1, 0, 0, 0, 0xFFFFFFFF, buf);
        renderXPosItem(sprite2, 0, 0, 0, color, buf);
        tes.func_78381_a();

        buf.func_181668_a(GL11.GL_QUADS, DefaultVertexFormats.field_176599_b);
        renderXNegItem(sprite1, 0, 0, 0, 0xFFFFFFFF, buf);
        renderXNegItem(sprite2, 0, 0, 0, color, buf);
        tes.func_78381_a();
    }

    @Override
    public void renderBrightness(IBlockState state, float brightness) {
        renderBlockAsItem(new ItemStack(state.func_177230_c(), 1, state.func_177230_c().func_176201_c(state)));
    }

    @Override
    public void registerTextures(TextureMap map) {
    }

    public void setRenderBoundsFromBlock1(GT_Generic_Block block) {
        this.minX = block.minX;
        this.minY = block.minY;
        this.minZ = block.minZ;
        this.maxX = block.maxX;
        this.maxY = block.maxY;
        this.maxZ = block.maxZ;
    }

    public void setBlockBounds(float minX, float minY, float minZ, float maxX, float maxY, float maxZ) {
        this.minX = minX;
        this.minY = minY;
        this.minZ = minZ;
        this.maxX = maxX;
        this.maxY = maxY;
        this.maxZ = maxZ;
    }

    public void renderYPosItem(TextureAtlasSprite sprite, double x, double y, double z, int color, VertexBuffer buffer) {
        float r = ((color >> 16) & 0xFF) / 255.0f;
        float g = ((color >> 8) & 0xFF) / 255.0f;
        float b = ((color) & 0xFF) / 255.0f;
        float a = ((color >> 24) & 0xFF) / 255.0f;

        double minU = sprite.func_94209_e();
        double minV = sprite.func_94206_g();
        double maxU = sprite.func_94212_f();
        double maxV = sprite.func_94210_h();

        buffer.func_181662_b(x + minX, y + maxY, z + maxZ).func_181666_a(r, g, b, a).func_187315_a(minU, maxV).func_181663_c(0.0f, 1.0f, 0.0f).func_181675_d();
        buffer.func_181662_b(x + maxX, y + maxY, z + maxZ).func_181666_a(r, g, b, a).func_187315_a(maxU, maxV).func_181663_c(0.0f, 1.0f, 0.0f).func_181675_d();
        buffer.func_181662_b(x + maxX, y + maxY, z + minZ).func_181666_a(r, g, b, a).func_187315_a(maxU, minV).func_181663_c(0.0f, 1.0f, 0.0f).func_181675_d();
        buffer.func_181662_b(x + minX, y + maxY, z + minZ).func_181666_a(r, g, b, a).func_187315_a(minU, minV).func_181663_c(0.0f, 1.0f, 0.0f).func_181675_d();

    }

    public void renderYNegItem(TextureAtlasSprite sprite, double x, double y, double z, int color, VertexBuffer buffer) {
        float r = ((color >> 16) & 0xFF) / 255.0f;
        float g = ((color >> 8) & 0xFF) / 255.0f;
        float b = ((color) & 0xFF) / 255.0f;
        float a = ((color >> 24) & 0xFF) / 255.0f;

        double minU = sprite.func_94209_e();
        double minV = sprite.func_94206_g();
        double maxU = sprite.func_94212_f();
        double maxV = sprite.func_94210_h();

        buffer.func_181662_b(x + minX, y + minY, z + minZ).func_181666_a(r, g, b, a).func_187315_a(minU, minV).func_181663_c(0.0f, -1.0f, 0.0f).func_181675_d();
        buffer.func_181662_b(x + maxX, y + minY, z + minZ).func_181666_a(r, g, b, a).func_187315_a(maxU, minV).func_181663_c(0.0f, -1.0f, 0.0f).func_181675_d();
        buffer.func_181662_b(x + maxX, y + minY, z + maxZ).func_181666_a(r, g, b, a).func_187315_a(maxU, maxV).func_181663_c(0.0f, -1.0f, 0.0f).func_181675_d();
        buffer.func_181662_b(x + minX, y + minY, z + maxZ).func_181666_a(r, g, b, a).func_187315_a(minU, maxV).func_181663_c(0.0f, -1.0f, 0.0f).func_181675_d();
    }

    public void renderZNegItem(TextureAtlasSprite sprite, double x, double y, double z, int color, VertexBuffer buffer) {
        float r = ((color >> 16) & 0xFF) / 255.0f;
        float g = ((color >> 8) & 0xFF) / 255.0f;
        float b = ((color) & 0xFF) / 255.0f;
        float a = ((color >> 24) & 0xFF) / 255.0f;


        double minU = sprite.func_94209_e();
        double minV = sprite.func_94206_g();
        double maxU = sprite.func_94212_f();
        double maxV = sprite.func_94210_h();

        buffer.func_181662_b(x + minX, y + minY, z + minZ).func_181666_a(r, g, b, a).func_187315_a(flip ? maxU : minU, flip ? maxV : minV).func_181663_c(0.0f, 0.0f, -1.0f).func_181675_d();
        buffer.func_181662_b(x + minX, y + maxY, z + minZ).func_181666_a(r, g, b, a).func_187315_a(flip ? maxU : minU, flip ? minV : maxV).func_181663_c(0.0f, 0.0f, -1.0f).func_181675_d();
        buffer.func_181662_b(x + maxX, y + maxY, z + minZ).func_181666_a(r, g, b, a).func_187315_a(flip ? minU : maxU, flip ? minV : maxV).func_181663_c(0.0f, 0.0f, -1.0f).func_181675_d();
        buffer.func_181662_b(x + maxX, y + minY, z + minZ).func_181666_a(r, g, b, a).func_187315_a(flip ? minU : maxU, flip ? maxV : minV).func_181663_c(0.0f, 0.0f, -1.0f).func_181675_d();
    }

    public void renderZPosItem(TextureAtlasSprite sprite, double x, double y, double z, int color, VertexBuffer buffer) {
        float r = ((color >> 16) & 0xFF) / 255.0f;
        float g = ((color >> 8) & 0xFF) / 255.0f;
        float b = ((color) & 0xFF) / 255.0f;
        float a = ((color >> 24) & 0xFF) / 255.0f;


        double minU = sprite.func_94209_e();
        double minV = sprite.func_94206_g();
        double maxU = sprite.func_94212_f();
        double maxV = sprite.func_94210_h();

        buffer.func_181662_b(x + maxX, y + minY, z + maxZ).func_181666_a(r, g, b, a).func_187315_a(flip ? minU : maxU, flip ? maxV : minV).func_181663_c(0.0f, 0.0f, 1.0f).func_181675_d();
        buffer.func_181662_b(x + maxX, y + maxY, z + maxZ).func_181666_a(r, g, b, a).func_187315_a(flip ? minU : maxU, flip ? minV : maxV).func_181663_c(0.0f, 0.0f, 1.0f).func_181675_d();
        buffer.func_181662_b(x + minX, y + maxY, z + maxZ).func_181666_a(r, g, b, a).func_187315_a(flip ? maxU : minU, flip ? minV : maxV).func_181663_c(0.0f, 0.0f, 1.0f).func_181675_d();
        buffer.func_181662_b(x + minX, y + minY, z + maxZ).func_181666_a(r, g, b, a).func_187315_a(flip ? maxU : minU, flip ? maxV : minV).func_181663_c(0.0f, 0.0f, 1.0f).func_181675_d();
    }

    public void renderXNegItem(TextureAtlasSprite sprite, double x, double y, double z, int color, VertexBuffer buffer) {
        float r = ((color >> 16) & 0xFF) / 255.0f;
        float g = ((color >> 8) & 0xFF) / 255.0f;
        float b = ((color) & 0xFF) / 255.0f;
        float a = ((color >> 24) & 0xFF) / 255.0f;

        double minU = sprite.func_94209_e();
        double minV = sprite.func_94206_g();
        double maxU = sprite.func_94212_f();
        double maxV = sprite.func_94210_h();

        buffer.func_181662_b(x + minX, y + minY, z + maxZ).func_181666_a(r, g, b, a).func_187315_a(flip ? minU : maxU, flip ? maxV : minV).func_181663_c(-1.0f, 0.0f, 0.0f).func_181675_d();
        buffer.func_181662_b(x + minX, y + maxY, z + maxZ).func_181666_a(r, g, b, a).func_187315_a(flip ? minU : maxU, flip ? minV : maxV).func_181663_c(-1.0f, 0.0f, 0.0f).func_181675_d();
        buffer.func_181662_b(x + minX, y + maxY, z + minZ).func_181666_a(r, g, b, a).func_187315_a(flip ? maxU : minU, flip ? minV : maxV).func_181663_c(-1.0f, 0.0f, 0.0f).func_181675_d();
        buffer.func_181662_b(x + minX, y + minY, z + minZ).func_181666_a(r, g, b, a).func_187315_a(flip ? maxU : minU, flip ? maxV : minV).func_181663_c(-1.0f, 0.0f, 0.0f).func_181675_d();
    }

    public void renderXPosItem(TextureAtlasSprite sprite, double x, double y, double z, int color, VertexBuffer buffer) {
        float r = ((color >> 16) & 0xFF) / 255.0f;
        float g = ((color >> 8) & 0xFF) / 255.0f;
        float b = ((color) & 0xFF) / 255.0f;
        float a = ((color >> 24) & 0xFF) / 255.0f;

        double minU = sprite.func_94209_e();
        double minV = sprite.func_94206_g();
        double maxU = sprite.func_94212_f();
        double maxV = sprite.func_94210_h();

        buffer.func_181662_b(x + maxX, y + minY, z + minZ).func_181666_a(r, g, b, a).func_187315_a(flip ? maxU : minU, flip ? maxV : minV).func_181663_c(1.0f, 0.0f, 0.0f).func_181675_d();
        buffer.func_181662_b(x + maxX, y + maxY, z + minZ).func_181666_a(r, g, b, a).func_187315_a(flip ? maxU : minU, flip ? minV : maxV).func_181663_c(1.0f, 0.0f, 0.0f).func_181675_d();
        buffer.func_181662_b(x + maxX, y + maxY, z + maxZ).func_181666_a(r, g, b, a).func_187315_a(flip ? minU : maxU, flip ? minV : maxV).func_181663_c(1.0f, 0.0f, 0.0f).func_181675_d();
        buffer.func_181662_b(x + maxX, y + minY, z + maxZ).func_181666_a(r, g, b, a).func_187315_a(flip ? minU : maxU, flip ? maxV : minV).func_181663_c(1.0f, 0.0f, 0.0f).func_181675_d();
    }


    public void renderYPos(TextureAtlasSprite sprite, double x, double y, double z, int color, int lightmap, VertexBuffer buffer) {
        float r = ((color >> 16) & 0xFF) / 255.0f;
        float g = ((color >> 8) & 0xFF) / 255.0f;
        float b = ((color) & 0xFF) / 255.0f;
        float a = ((color >> 24) & 0xFF) / 255.0f;


        int light1 = Math.max(0, (lightmap >> 16 & 65535) - 26);
        int light2 = Math.max(0, (lightmap & 65535) - 26);

        double minU = sprite.func_94209_e();
        double minV = sprite.func_94206_g();
        double maxU = sprite.func_94212_f();
        double maxV = sprite.func_94210_h();

        buffer.func_181662_b(x + minX, y + maxY, z + maxZ).func_181666_a(r, g, b, a).func_187315_a(minU, maxV).func_187314_a(light1, light2).func_181675_d();
        buffer.func_181662_b(x + maxX, y + maxY, z + maxZ).func_181666_a(r, g, b, a).func_187315_a(maxU, maxV).func_187314_a(light1, light2).func_181675_d();
        buffer.func_181662_b(x + maxX, y + maxY, z + minZ).func_181666_a(r, g, b, a).func_187315_a(maxU, minV).func_187314_a(light1, light2).func_181675_d();
        buffer.func_181662_b(x + minX, y + maxY, z + minZ).func_181666_a(r, g, b, a).func_187315_a(minU, minV).func_187314_a(light1, light2).func_181675_d();

    }

    public void renderYNeg(TextureAtlasSprite sprite, double x, double y, double z, int color, int lightmap, VertexBuffer buffer) {
        float r = ((color >> 16) & 0xFF) / 255.0f;
        float g = ((color >> 8) & 0xFF) / 255.0f;
        float b = ((color) & 0xFF) / 255.0f;
        float a = ((color >> 24) & 0xFF) / 255.0f;


        int light1 = Math.max(0, (lightmap >> 16 & 65535) - 26);
        int light2 = Math.max(0, (lightmap & 65535) - 26);

        double minU = sprite.func_94209_e();
        double minV = sprite.func_94206_g();
        double maxU = sprite.func_94212_f();
        double maxV = sprite.func_94210_h();

        buffer.func_181662_b(x + minX, y + minY, z + minZ).func_181666_a(r, g, b, a).func_187315_a(minU, minV).func_187314_a(light1, light2).func_181675_d();
        buffer.func_181662_b(x + maxX, y + minY, z + minZ).func_181666_a(r, g, b, a).func_187315_a(maxU, minV).func_187314_a(light1, light2).func_181675_d();
        buffer.func_181662_b(x + maxX, y + minY, z + maxZ).func_181666_a(r, g, b, a).func_187315_a(maxU, maxV).func_187314_a(light1, light2).func_181675_d();
        buffer.func_181662_b(x + minX, y + minY, z + maxZ).func_181666_a(r, g, b, a).func_187315_a(minU, maxV).func_187314_a(light1, light2).func_181675_d();
    }

    public void renderZNeg(TextureAtlasSprite sprite, double x, double y, double z, int color, int lightmap, VertexBuffer buffer) {
        float r = ((color >> 16) & 0xFF) / 255.0f;
        float g = ((color >> 8) & 0xFF) / 255.0f;
        float b = ((color) & 0xFF) / 255.0f;
        float a = ((color >> 24) & 0xFF) / 255.0f;


        int light1 = Math.max(0, (lightmap >> 16 & 65535) - 26);
        int light2 = Math.max(0, (lightmap & 65535) - 26);

        double minU = sprite.func_94209_e();
        double minV = sprite.func_94206_g();
        double maxU = sprite.func_94212_f();
        double maxV = sprite.func_94210_h();

        buffer.func_181662_b(x + minX, y + minY, z + minZ).func_181666_a(r, g, b, a).func_187315_a(flip ? maxU : minU, flip ? maxV : minV).func_187314_a(light1, light2).func_181675_d();
        buffer.func_181662_b(x + minX, y + maxY, z + minZ).func_181666_a(r, g, b, a).func_187315_a(flip ? maxU : minU, flip ? minV : maxV).func_187314_a(light1, light2).func_181675_d();
        buffer.func_181662_b(x + maxX, y + maxY, z + minZ).func_181666_a(r, g, b, a).func_187315_a(flip ? minU : maxU, flip ? minV : maxV).func_187314_a(light1, light2).func_181675_d();
        buffer.func_181662_b(x + maxX, y + minY, z + minZ).func_181666_a(r, g, b, a).func_187315_a(flip ? minU : maxU, flip ? maxV : minV).func_187314_a(light1, light2).func_181675_d();
    }

    public void renderZPos(TextureAtlasSprite sprite, double x, double y, double z, int color, int lightmap, VertexBuffer buffer) {
        float r = ((color >> 16) & 0xFF) / 255.0f;
        float g = ((color >> 8) & 0xFF) / 255.0f;
        float b = ((color) & 0xFF) / 255.0f;
        float a = ((color >> 24) & 0xFF) / 255.0f;


        int light1 = Math.max(0, (lightmap >> 16 & 65535) - 26);
        int light2 = Math.max(0, (lightmap & 65535) - 26);

        double minU = sprite.func_94209_e();
        double minV = sprite.func_94206_g();
        double maxU = sprite.func_94212_f();
        double maxV = sprite.func_94210_h();

        buffer.func_181662_b(x + maxX, y + minY, z + maxZ).func_181666_a(r, g, b, a).func_187315_a(flip ? minU : maxU, flip ? maxV : minV).func_187314_a(light1, light2).func_181675_d();
        buffer.func_181662_b(x + maxX, y + maxY, z + maxZ).func_181666_a(r, g, b, a).func_187315_a(flip ? minU : maxU, flip ? minV : maxV).func_187314_a(light1, light2).func_181675_d();
        buffer.func_181662_b(x + minX, y + maxY, z + maxZ).func_181666_a(r, g, b, a).func_187315_a(flip ? maxU : minU, flip ? minV : maxV).func_187314_a(light1, light2).func_181675_d();
        buffer.func_181662_b(x + minX, y + minY, z + maxZ).func_181666_a(r, g, b, a).func_187315_a(flip ? maxU : minU, flip ? maxV : minV).func_187314_a(light1, light2).func_181675_d();
    }

    public void renderXNeg(TextureAtlasSprite sprite, double x, double y, double z, int color, int lightmap, VertexBuffer buffer) {
        float r = ((color >> 16) & 0xFF) / 255.0f;
        float g = ((color >> 8) & 0xFF) / 255.0f;
        float b = ((color) & 0xFF) / 255.0f;
        float a = ((color >> 24) & 0xFF) / 255.0f;


        int light1 = Math.max(0, (lightmap >> 16 & 65535) - 26);
        int light2 = Math.max(0, (lightmap & 65535) - 26);

        double minU = sprite.func_94209_e();
        double minV = sprite.func_94206_g();
        double maxU = sprite.func_94212_f();
        double maxV = sprite.func_94210_h();

        buffer.func_181662_b(x + minX, y + minY, z + maxZ).func_181666_a(r, g, b, a).func_187315_a(flip ? minU : maxU, flip ? maxV : minV).func_187314_a(light1, light2).func_181675_d();
        buffer.func_181662_b(x + minX, y + maxY, z + maxZ).func_181666_a(r, g, b, a).func_187315_a(flip ? minU : maxU, flip ? minV : maxV).func_187314_a(light1, light2).func_181675_d();
        buffer.func_181662_b(x + minX, y + maxY, z + minZ).func_181666_a(r, g, b, a).func_187315_a(flip ? maxU : minU, flip ? minV : maxV).func_187314_a(light1, light2).func_181675_d();
        buffer.func_181662_b(x + minX, y + minY, z + minZ).func_181666_a(r, g, b, a).func_187315_a(flip ? maxU : minU, flip ? maxV : minV).func_187314_a(light1, light2).func_181675_d();
    }

    public void renderXPos(TextureAtlasSprite sprite, double x, double y, double z, int color, int lightmap, VertexBuffer buffer) {
        float r = ((color >> 16) & 0xFF) / 255.0f;
        float g = ((color >> 8) & 0xFF) / 255.0f;
        float b = ((color) & 0xFF) / 255.0f;
        float a = ((color >> 24) & 0xFF) / 255.0f;

        int light1 = Math.max(0, (lightmap >> 16 & 65535) - 26);
        int light2 = Math.max(0, (lightmap & 65535) - 26);

        double minU = sprite.func_94209_e();
        double minV = sprite.func_94206_g();
        double maxU = sprite.func_94212_f();
        double maxV = sprite.func_94210_h();

        buffer.func_181662_b(x + maxX, y + minY, z + minZ).func_181666_a(r, g, b, a).func_187315_a(flip ? maxU : minU, flip ? maxV : minV).func_187314_a(light1, light2).func_181675_d();
        buffer.func_181662_b(x + maxX, y + maxY, z + minZ).func_181666_a(r, g, b, a).func_187315_a(flip ? maxU : minU, flip ? minV : maxV).func_187314_a(light1, light2).func_181675_d();
        buffer.func_181662_b(x + maxX, y + maxY, z + maxZ).func_181666_a(r, g, b, a).func_187315_a(flip ? minU : maxU, flip ? minV : maxV).func_187314_a(light1, light2).func_181675_d();
        buffer.func_181662_b(x + maxX, y + minY, z + maxZ).func_181666_a(r, g, b, a).func_187315_a(flip ? minU : maxU, flip ? maxV : minV).func_187314_a(light1, light2).func_181675_d();
    }

}
